@@ -14,9 +14,12 @@ from .subbasin_multiplier import SubbasinMultiplier
 from .outlet_drainage import OutletDrainage
 from ...delineation.structure import Structure
 from ...names import Names
+from ...bmp.bmp_manure_adjustment import ManureAdjustmentBMPType, ManureAdjustmentBMP
 from ...bmp.bmp_areal_manure_feedlot import ArealBMPManureFeedlot
 from ...bmp.bmp_areal_manure_storage import ArealBMPManureStorage
 from ...bmp.bmp_reach_manure_catch_basin import ReachBMPManureCatchBasin
+from ...bmp.bmp_reach_point_source import ReachBMPPointSource
+from ...bmp.bmp_reach_grass_waterway import ReachBMPGrassWaterWay
 from ...bmp.bmp_reach_reservoir import ReachBMPReservoir
 from ...bmp.bmp_reach_wetland import ReachBMPWetland
 from ...bmp.bmp_type import BMPType
@@ -27,41 +30,20 @@ from ...bmp.bmp_structure_wascob import StructureBMPWascob
 from ...bmp.bmp_structure_tile_drain import StructureBMPTileDrain
 from ...bmp.bmp_offsite_wintering import BMPOffsiteWintering
 from ...bmp.crop_rotation.crop_rotation import CropRotation
+from ...bmp.crop_rotation.single_crop import SingleCrop
 from .bmp_01_point_source import PointSource
 from .bmp_02_flow_diversion import FlowDiversion
 from .bmp_03_reservoir import Reservoir
 from .bmp_05_riparian_buffer import RiparianBuffer
-# from .bmp_06_grass_waterway import GrassWaterWay
-# from .bmp_07_vegetation_filter_strip import FilterStrip
+from .bmp_07_vegetation_filter_strip import VegetationFilterStrip
 from .bmp_09_isolated_wetland import Wetland
-# from .bmp_12_crop_management import CropManagement, CropParameter
-# from .bmp_14_tillage_management import TillageManagement, TillageParameter
-# from .bmp_15_fertilizer_management import FertlizerManagement, FertilizerParameter
-# from .bmp_16_grazing_management import GrazingManagement
-# from .bmp_18_irrigation_management import irrigation_management, irrigation_parameter
 from .bmp_19_tile_drain_management import TileDrainParameter
-# from .bmp_21_manure_incorporation_within_48h import ManureIncorporationWithin48hManagemet
-# from .bmp_22_manure_application_setback import ManureApplicationSetbackManagement
-# from .bmp_23_no_application_on_snow import ManureNoApplicaitonOnSnowManagement
-# from .bmp_24_spring_application_rather_than_fall_application import ManureSpringApplicationRatherThanFallApplicationManagement
-# from .bmp_25_applicaiton_based_on_soil_nitrogen_limit import ManureApplicationBasedOnSoliNitrogenLimitManagement
-# from .bmp_26_applicaiton_based_on_soil_phosphorous_limit import ManureApplicationBasedOnPhosphorousLimitManagement
 from .bmp_27_manure_storage import ManureStorageParameter
 from .bmp_28_manure_catch_basin import ManureCatchBasinParameter
 from .bmp_29_manure_feedlot import ManureFeedlot, ManureFeedlotManagement
-# from .bmp_30_marginal_crop_management import MarginalCropManagement
-# from .bmp_31_marginal_fertilizer_management import MarginalFertilizerManagement
-# from .bmp_32_marginal_tillage_management import MarginalTillageManagement
-# from .bmp_33_wintering_site_management import WinteringSiteManagement, WinteringSiteParameter
-# from .bmp_34_pasture_crop_management import PastureCropManagement
-# from .bmp_35_pasture_fertilizer_management import PastureFertilizerManagement
-# from .bmp_36_pasture_tillage_management import Pasture_tillage_management
-# from .bmp_37_pasture_grazing_management import PastureGrazingManagement, PastureGrazingParameter
-# from .bmp_38_dugout import DugoutParameter
 from .bmp_39_offsite_watering import OffsiteWateringParameter
 from .bmp_40_managed_access_including_fencing import ManagedAccessIncludingFencingParameter
 from .bmp_41_wascob import Wascob
-#from .bmp_42_water_use import 
 import os
 import gc
 
@@ -114,10 +96,16 @@ class BMPDatabase(DatabaseBase):
         self.__create_bmp_manure_catchbasin(outputs)
         self.__create_bmp_manure_storage(outputs)
 
-        self.__create_bmp_reservoir(outputs,reservoir_flow_routing, reservoir_flow_data_folder)
-        self.__create_bmp_wetland(outputs)
+        reach_bmps_subbasins = {}
+        reach_bmps_subbasins[BMPType.BMP_TYPE_POINTSOURCE] = self.__create_bmp_point_source(outputs)
+        reach_bmps_subbasins[BMPType.BMP_TYPE_RESERVOIR] = self.__create_bmp_reservoir(outputs,reservoir_flow_routing, reservoir_flow_data_folder)
+        reach_bmps_subbasins[BMPType.BMP_TYPE_GRASSWATERWAY] = self.__create_bmp_grass_waterway(outputs)
+        reach_bmps_subbasins[BMPType.BMP_TYPE_WETLAND] = self.__create_bmp_wetland(outputs)
+        
+        
         self.__create_bmp_dugout(outputs)        
         self.__create_bmp_riparian_buffer(outputs)
+        self.__create_bmp_filter_strip(outputs)
 
         tile_drain = self.__create_bmp_tile_drain(outputs)
         self.__create_bmp_wascob(outputs, tile_drain)
@@ -128,7 +116,14 @@ class BMPDatabase(DatabaseBase):
         #all reach bmps should be added before this
         self.__create_reach_parameter(outputs)
         self.__create_reach_lookup()
-        self.__create_bmp_reach_bmp(outputs)
+        self.__create_bmp_reach_bmp(outputs, reach_bmps_subbasins)
+
+        #create marginal and pasture crop bmps
+        self.__create_bmp_marginal_crop(outputs)
+        self.__create_bmp_pasture_crop(outputs)
+
+        #create manure adjustment
+        self.__create_bmp_manure_adjustment(outputs)
 
         #create bmp_scenarios table
         self.__create_bmp_scenarios(outputs)
@@ -333,6 +328,11 @@ class BMPDatabase(DatabaseBase):
         if df is not None:
             self.save_table(Names.bmp_table_name_riparian_buffer, df, RiparianBuffer.column_types())
 
+    def __create_bmp_filter_strip(self, outputs:Outputs):
+        df = outputs.filter_strip_parameter_df
+        if df is not None:
+            self.save_table(Names.bmp_table_name_filter_strip, df, VegetationFilterStrip.column_types())
+
 #endregion
 
 #region Manure Management BMPs
@@ -398,51 +398,72 @@ class BMPDatabase(DatabaseBase):
 
 #region Reach BMPs
 
-    def __create_bmp_reach_bmp(self, outputs:Outputs):
+    def __create_bmp_reach_bmp(self, outputs:Outputs, reach_bmp_subbasins:dict):
         """create reach_bmp table"""
 
         logger.info("Creating reach_bmp table ... ")
 
-        reach_bmps = {} 
-
-        #wetland      
-        if "wetland" in outputs.structures:
-            reach_bmps[BMPType.BMP_TYPE_WETLAND] = ReachBMPWetland(outputs.structures["wetland"].boundary_processed_vector, outputs.subbasin_raster).subbasins
-        
-        #reservoir
-        if outputs.inputs.reservoir_vector is not None:
-            reach_bmps[BMPType.BMP_TYPE_RESERVOIR] = ReachBMPReservoir(outputs.inputs.reservoir_vector, outputs.subbasin_raster).subbasins
-        
-        #catch basin
-        if outputs.inputs.catchbasin_vector is not None:
-            reach_bmps[BMPType.BMP_TYPE_MANURE_CATCHBASIN] = ReachBMPManureCatchBasin(outputs.inputs.catchbasin_vector,
-                                    outputs.subbasin_raster,
-                                    outputs.reach_parameter_df).subbasins
-
-        if len(reach_bmps) <= 0:
-            logger.info("There is no reach bmp, existing ... ")
-            return
-
         #get all subbasins
         subbasin_ids = range(1, int(RasterExtension.get_max_value(outputs.subbasin_raster)) + 1)
 
         #save the table
-        df = ReachBMP.create_reach_bmp_df(subbasin_ids, reach_bmps)
+        df = ReachBMP.create_reach_bmp_df(subbasin_ids, reach_bmp_subbasins)
         self.save_table(Names.bmp_table_name_reach_bmp, df)
 
-    def __create_bmp_wetland(self, outputs:Outputs):
+    def __create_bmp_wetland(self, outputs:Outputs)->dict:
         if "wetland" not in outputs.structures:
-            return
+            return {}
         
-        logger.info("Creating wetland ... ")
+        logger.info("Creating wetland parameter table ... ")
         Session = sessionmaker(bind=self.engine)
         with Session() as session:
             session.add_all([Wetland(struc_attribute) for struc_attribute in outputs.structures["wetland"].attributes.values()])
             session.commit()
 
+        return ReachBMPWetland(outputs.structures["wetland"].boundary_processed_vector, outputs.subbasin_raster).subbasins
+
+    def __create_bmp_grass_waterway(self, outputs:Outputs)->dict:
+        """
+        populate grass waterway parameter table
+        """
+
+        if outputs.inputs.grass_waterway_vector is None:
+            return {}
+        
+        logger.info("Creating grass waterway parameter table ... ")
+        grass_waterway = ReachBMPGrassWaterWay(outputs.inputs.grass_waterway_vector, 
+                                             outputs.subbasin_raster,
+                                             outputs.flow_direction_raster,
+                                             outputs.reach_parameter_df)
+
+        Session = sessionmaker(bind=self.engine)
+        with Session() as session:
+            session.add_all(grass_waterway.grass_waterways)
+            session.commit()  
+        
+        return grass_waterway.id_subbasins
+
+    def __create_bmp_point_source(self, outputs:Outputs)->dict:
+        """
+        populate point source parameter table
+        """
+
+        if outputs.inputs.point_source_vector is None:
+            return {}
+        
+        logger.info("Creating point source parameter table ... ")
+        point_source = ReachBMPPointSource(outputs.inputs.point_source_vector, outputs.subbasin_raster)
+
+        Session = sessionmaker(bind=self.engine)
+        with Session() as session:
+            session.add_all(point_source.point_sources)
+            session.commit()  
+
+        return point_source.subbasins
+
     def __create_bmp_reservoir(self, outputs:Outputs, 
                            flow_method:str, 
-                           flow_data_folder:str):
+                           flow_data_folder:str)->dict:
         """
         populate reservoir parameter table
         """
@@ -456,10 +477,11 @@ class BMPDatabase(DatabaseBase):
 
         for res in reservoir.reservoirs:
             res.METHOD = flow_method
-            logger.info(f"Importing {res.FILE}.csv ...")
-            file_path = os.path.join(flow_data_folder, f"{res.FILE}.csv")
-            if os.path.exists(file_path):
-                self.save_table(res.FILE, pd.read_csv(file_path))
+            if flow_method in ReachBMPReservoir.FLOW_ROUTING_METHODS_REQUIRING_EXTERNAL_DATA and flow_data_folder is not None and res.FILE is not None and len(res.FILE) > 0:
+                logger.info(f"Importing {res.FILE}.csv ...")
+                file_path = os.path.join(flow_data_folder, f"{res.FILE}.csv")
+                if os.path.exists(file_path):
+                    self.save_table(res.FILE, pd.read_csv(file_path))
 
         Session = sessionmaker(bind=self.engine)
         with Session() as session:
@@ -488,6 +510,14 @@ class BMPDatabase(DatabaseBase):
         logger.info("Creating bmp_secenarios table ... ")
 
         bmp_types = outputs.inputs.bmp_types
+        if outputs.marginal_crop_land_separated_field_raster is not None:
+            bmp_types.append(BMPType.BMP_TYPE_CROP_MAR)
+            bmp_types.append(BMPType.BMP_TYPE_FERTILIZER_MAR)
+            bmp_types.append(BMPType.BMP_TYPE_TILLAGE_MAR)
+        if outputs.pasture_crop_land_separated_field_raster is not None:
+            bmp_types.append(BMPType.BMP_TYPE_CROP_PS)
+            bmp_types.append(BMPType.BMP_TYPE_FERTILIZER_PS)
+            bmp_types.append(BMPType.BMP_TYPE_TILLAGE_PS)           
         if include_crop_rotation:
             bmp_types.append(BMPType.BMP_TYPE_CROP)
             bmp_types.append(BMPType.BMP_TYPE_FERTILIZER)
@@ -498,20 +528,69 @@ class BMPDatabase(DatabaseBase):
         df = BMP.generate_bmp_scenarios_df(bmp_types)
         self.save_table(Names.bmp_table_name_scenarios, df)
 
+#region Manure Adjustment
+
+    def __create_bmp_manure_adjustment(self, outputs:Outputs):
+        for type in ManureAdjustmentBMPType:
+            raster = outputs.get_manure_adjustment_raster(type)
+
+            if raster is not None:
+                logger.info(f"Creating manure adjustment bmp: {type} ... ")
+
+                bmp = ManureAdjustmentBMP(raster, type)
+                self.save_table(bmp.table_name, bmp.manure_adjustment_df)               
+
+
+#endregion
+
 #region Crop Rotation
 
+    def __create_bmp_pasture_crop(self, outputs:Outputs):
+        if outputs.pasture_crop_land_separated_field_raster is None:
+            return
+        
+        logger.info("Creating pasture crop bmps ... ")
+
+        crop = SingleCrop(outputs.pasture_crop_land_separated_field_raster, outputs.pasture_crop_land_grass_type)
+
+        logger.info("   Crop Management ... ")
+        self.save_table(Names.bmp_table_name_pasture_crop_management, crop.crop_management_df)
+
+        logger.info("   Fertilizer Management ... ")
+        self.save_table(Names.bmp_table_name_pasture_fertilizer_management, crop.fertilizer_management_df)
+
+        logger.info("   Tillage Management ... ")
+        self.save_table(Names.bmp_table_name_pasture_tillage_management, crop.tillage_management_df)
+
+    def __create_bmp_marginal_crop(self, outputs:Outputs):
+        if outputs.marginal_crop_land_separated_field_raster is None:
+            return
+        
+        logger.info("Creating marginal crop bmps ... ")
+
+        crop = SingleCrop(outputs.marginal_crop_land_separated_field_raster, outputs.marginal_crop_land_grass_type)
+
+        logger.info("   Crop Management ... ")
+        self.save_table(Names.bmp_table_name_marginal_crop_management, crop.crop_management_df)
+
+        logger.info("   Fertilizer Management ... ")
+        self.save_table(Names.bmp_table_name_marginal_fertilizer_management, crop.fertilizer_management_df)
+
+        logger.info("   Tillage Management ... ")
+        self.save_table(Names.bmp_table_name_marginal_tillage_management, crop.tillage_management_df)
+
     def update_crop_rotation_AAFC_crop_inventory(self, crop_inventory_folder:str, first_year:int, last_year:int, outputs:Outputs, include_grazing:bool = False):        
         logger.info("Updating crop rotation based on AAFC Crop Inventory ... ")
 
         rotation = CropRotation(outputs.field_clipped_vector, outputs.subbasin_raster,crop_inventory_folder, first_year, last_year)
         
-        logger.info("Crop Management ... ")
+        logger.info("   Crop Management ... ")
         self.save_table(Names.bmp_table_name_crop_management, rotation.crop_management_df)
 
-        logger.info("Fertilizer Management ... ")
+        logger.info("   Fertilizer Management ... ")
         self.save_table(Names.bmp_table_name_fertilizer_management, rotation.fertilizer_management_df)
 
-        logger.info("Tillage Management ... ")
+        logger.info("   Tillage Management ... ")
         self.save_table(Names.bmp_table_name_tillage_management, rotation.tillage_management_df)
 
         if include_grazing:
@@ -631,7 +710,7 @@ class BMPDatabase(DatabaseBase):
 
         #SubAreaLandUseType
         logger.info("Creating SubAreaLandUseType ...")
-        subarea_landuse_df = RasterExtension.get_overlay_area(outputs.subarea_raster, outputs.mapped_landuse_raster, 
+        subarea_landuse_df = RasterExtension.get_overlay_area(outputs.subarea_raster, outputs.mapped_landuse_final_raster, 
                               "SubareaId", "LanduseTypeId", "Area")[["SubareaId", "LanduseTypeId", "Area"]]
         self.save_table(Names.bmp_table_name_subarea_landuse, subarea_landuse_df)     
 
@@ -648,6 +727,10 @@ class BMPDatabase(DatabaseBase):
         self.__create_subarea_structure_lookup_table(subarea_area_df, outputs.subarea_raster, outputs.riparian_buffer_parts_raster, Names.bmp_table_name_subarea_riparian_buffer_lookup, override)
         self.__create_subarea_structure_lookup_table(subarea_area_df, outputs.subarea_raster, outputs.riparian_buffer_drainage_area_raster, Names.bmp_table_name_subarea_riparian_buffer_drainage_lookup, override)
         
+        #filter strip
+        self.__create_subarea_structure_lookup_table(subarea_area_df, outputs.subarea_raster, outputs.filter_strip_parts_raster, Names.bmp_table_name_subarea_vegetative_filter_strip_lookup, override)
+        self.__create_subarea_structure_lookup_table(subarea_area_df, outputs.subarea_raster, outputs.filter_strip_drainage_area_raster, Names.bmp_table_name_subarea_vegetative_filter_strip_drainage_lookup, override)
+
         #feedlot
         self.__create_subarea_structure_lookup_table(subarea_area_df, outputs.subarea_raster, outputs.feedlot_raster, Names.bmp_table_name_subarea_feedlot_lookup, override)
         self.__create_subarea_structure_lookup_table(subarea_area_df, outputs.subarea_raster, outputs.feedlot_drainage_area_raster, Names.bmp_table_name_subarea_feedlot_drainage_lookup, override)
