@@ -20,6 +20,9 @@ class imWEBs:
         """
         self.model_config.delineate_watershed()
 
+    def generate_pour_points_based_on_threshold_and_structures(self):
+        self.model_config.generate_pour_points_based_on_threshold_and_structures()
+
     def generate_parameters(self):
         """
         watershed delineation
@@ -35,6 +38,9 @@ class imWEBs:
     def generate_scenario(self):
         self.scenario_config.generate_model_structure()
 
+    def generate_subarea_parameter_database(self):
+        self.scenario_config.generate_parameter_subarea_database()
+
     def generate_all(self):
         self.delineate_watershed()
         self.generate_parameters()
