@@ -45,6 +45,9 @@ class Names:
     bmp_table_name_manure_feed_lot_management = "non_structure_bmp_manure_feed_lot_management"
     bmp_table_name_wintering_site_parameter = "non_structure_bmp_wintering_site_parameter"
     bmp_table_name_wintering_site_management = "non_structure_bmp_wintering_site_management"
+    bmp_table_name_marginal_crop_management = "non_structure_bmp_marginal_crop_management"
+    bmp_table_name_marginal_fertilizer_management = "non_structure_bmp_marginal_fertilizer_management"
+    bmp_table_name_marginal_tillage_management = "non_structure_bmp_marginal_tillage_management"    
     bmp_table_name_pasture_crop_management = "non_structure_bmp_pasture_crop_management"
     bmp_table_name_pasture_fertilizer_management = "non_structure_bmp_pasture_fertilizer_management"
     bmp_table_name_pasture_tillage_management = "non_structure_bmp_pasture_tillage_management"    
@@ -89,6 +92,8 @@ class Names:
     bmp_table_name_subarea_wascob_drainage_lookup = "SubareaWascobDrainageLookup"
     bmp_table_name_subarea_tile_drain_drainage_lookup = "SubareaTileDrainLookup"
 
+    bmp_table_name_subarea_unit_climate_weight = "UnitClimateWeight"
+
 
 
     #
@@ -141,8 +146,11 @@ class Names:
     parameterSubareaDatabaseName = "parameter_subarea" + sqlite_extension
 
     # Lookup
-    soilLookupName = "soilLookup" + lookup_extension
-    landuseLookupName = "landuseLookup" + lookup_extension
+    soilLookupName = "soil_id_lookup" + lookup_extension
+    landuseLookupName = "landuse_id_lookup" + lookup_extension
+
+    # parameter
+    parameterSoilLookupName = "SoilLookup" + lookup_extension
 
     # DEM
     demName = "dem" + raster_extension
@@ -168,10 +176,12 @@ class Names:
     # mainStreamRasName = "mainStream" + raster_extension   
     streamMainRasName = "stream_main" + raster_extension
     streamMainShpName = "stream_main" + shapefile_extension 
+    streamThresholdRasName = "stream_threshold" + raster_extension
     streamNetworkRasName = "stream_network" + raster_extension
     streamNetworkShpName = "stream_network" + shapefile_extension
     streamPourPointShpName = "stream_pour_point" + shapefile_extension
     streamPourPointRasName = "stream_pour_point" + raster_extension
+    streamPourPointThresholdShpName = "stream_pour_point_threshold" + shapefile_extension
     streamOrderRasName = "stream_order" + raster_extension
 
     # Reach
@@ -245,7 +255,7 @@ class Names:
 
     # Landuse
     landuseName = "landuse" + raster_extension
-    landuseMappedName = "landuseMapped" + raster_extension
+    landuseMappedOriginalName = "landuseMappedOriginal" + raster_extension
 
     # Farm
     farmShpName = "farm" + shapefile_extension
@@ -255,13 +265,15 @@ class Names:
     # Field
     fieldShpName = "field" + shapefile_extension
     fieldRasName = fieldShpName.replace(shapefile_extension, raster_extension)
+    fieldOriginalRasName = "fieldOriginal" + raster_extension
     fieldClippedShpName = "fieldClipped" + shapefile_extension
     fiedWithOnlyAgricultureRasName = "fiedWithOnlyAgriCulture" + raster_extension
 
     # management unit
     subareaShpName = "subarea" + shapefile_extension
     subareaRasName = subareaShpName.replace(shapefile_extension, raster_extension)
-
+    subareaCentroidShpName = "subarea_centroid" + shapefile_extension
+    subareaCentroidCsvName = "subarea_centroid" + csv_extension
     #
 
 
@@ -418,15 +430,6 @@ class Names:
     winteringSiteShpName = "winteringSite" + shapefile_extension
     winteringSiteRasterName = winteringSiteShpName.replace(shapefile_extension,raster_extension)
 
-    #Pasture
-    pastureLandRasName = "PasCropland" + raster_extension
-    # pastureLandShpName = "PasCropland" + shapefile_extension
-    # pastureLandH5RasterName = "PasCropland_H5" + raster_extension
-
-    #Pasture grazing
-    pastureGrazingRasterName = "pastureGrazing" + raster_extension
-    pastureGrazingShpName = "pastureGrazing" + shapefile_extension
-
     #Manure Setback
     manure48hShpName = "manure_within_48h" + shapefile_extension
     manure48hRasName = manure48hShpName.replace(shapefile_extension, raster_extension)
@@ -460,9 +463,18 @@ class Names:
     # grazingOffsiteWateringRasterName = "GrazingOffsiteWatering" + raster_extension
 
     #Marginal Crop Land
-    marCroplandRasName = "MarCropland" + raster_extension
-    # marCroplandShpName = "MarCropland" + shapefile_extension
-    # marCroplandH5RasterName = "MarCropland_H5" + raster_extension
+    marginalCroplandShpName = "marginal_crop_land" + shapefile_extension
+    marginalCroplandOriginalFieldRasName = "marginal_crop_land_original_field" + raster_extension
+    marginalCroplandSeparatedFieldRasName = "marginal_crop_land_separated_field" + raster_extension
+
+    #Pasture crop land
+    pastureCropLandShpName = "pasture_crop_land" + shapefile_extension
+    pastureCropLandOriginalFieldRasName = "pasture_crop_land_original_field" + raster_extension
+    pastureCropLandSeparatedFieldRasName = "pasture_crop_land_separated_field" + raster_extension
+
+    #Pasture grazing
+    pastureGrazingRasterName = "pastureGrazing" + raster_extension
+    pastureGrazingShpName = "pastureGrazing" + shapefile_extension
 
     #standard names
     config_item_standard_name_lookup = {
@@ -480,6 +492,9 @@ class Names:
         "soil_lookup":soilLookupName,
         "landuse_lookup":landuseLookupName,
 
+        #parameters
+        "SoilLookup":parameterSoilLookupName,
+
         #db3 database
         "hydroclimate":hydroclimateDatabasename,
 
@@ -497,7 +512,7 @@ class Names:
         #structure_bmp
         "dugout_boundary_shapefile":dugoutShpName,        
         "riparian_buffer_shapefile":riparianBufferShpName,
-        "filter_strip_shapefile":filterStripRasterName,
+        "filter_strip_shapefile":filterStripShpName,
         "wascob_shapefile":wascobShpName,
         "tile_drain_boundary_shapefile":tileDrainShpName,
         "tile_drain_outlet_shapefile":tileDrainOutletShpName,
@@ -506,9 +521,44 @@ class Names:
         "manure_feedlot_boundary_shapefile": feedlotShpName,
         "manure_feedlot_outlet_shapefile": feedlotOutletShpName,
         "manure_storage_boundary_shapefile": manureStorageShpName,
-        "offsite_watering_shapefile": offsiteWinteringShpName
+        "offsite_watering_shapefile": offsiteWinteringShpName,
+
+        #margnial and pasture crop land
+        "marginal_crop_land_shapefile": marginalCroplandShpName,
+        "pasture_crop_land_shapefile": pastureCropLandShpName,
+
+        #manure adjustment bmp
+        "manure_adjustment_incorporation_within_48h_shapefile": manure48hShpName,
+        "manure_adjustment_application_setback_shapefile": manureSetbackShpName,
+        "manure_adjustment_no_application_on_snow_shapefile": manureNoOnSnowShpName,
+        "manure_adjustment_spring_rather_than_fall_shapefile": manureSpringShpName,
+        "manure_adjustment_based_on_n_limit_shapefile": manureNLimitShpName,
+        "manure_adjustment_based_on_p_limit_shapefile": manurePLimitShpName
     }
 
+    @staticmethod
+    def bmp_distributions()->list:
+        return [Names.remove_extension_from_raster_file(x) for x in 
+                [Names.fieldRasName, 
+                Names.marginalCroplandSeparatedFieldRasName, 
+                Names.pastureCropLandSeparatedFieldRasName,
+                Names.manureStorageRasName, 
+                Names.feedlotBoundaryProcessedRasName,
+                Names.riparianBufferPartRasterName,
+                Names.riparianBufferDrainageRasterName,
+                Names.filterStripPartRasterName,
+                Names.filterStripDrainageRasterName,
+                Names.dugoutRasName,
+                Names.wascobDrainageAreaRasName,
+                Names.tileDrainRasName,
+                Names.manure48hRasName,
+                Names.manureSetbackRasName,
+                Names.manureNoOnSnowRasName,
+                Names.manureSpringRasName,
+                Names.manureNLimitRasname,
+                Names.manurePLimitRasname]
+                ]
+
     @staticmethod
     def get_standard_file_name(item_name:str)->str:
         if item_name in Names.config_item_standard_name_lookup:
